"""The wfminpre commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - WFMInpre:BIT_Nr <NR1>
    - WFMInpre:BIT_Nr?
    - WFMInpre:BN_Fmt {RI|RP|FP}
    - WFMInpre:BN_Fmt?
    - WFMInpre:BYT_Nr <NR1>
    - WFMInpre:BYT_Nr?
    - WFMInpre:BYT_Or {LSB|MSB}
    - WFMInpre:BYT_Or?
    - WFMInpre:CENTERFREQuency <NR3>
    - WFMInpre:CENTERFREQuency?
    - WFMInpre:DOMain {TIMe|FREQuency}
    - WFMInpre:DOMain?
    - WFMInpre:ENCdg {ASCii|BINary}
    - WFMInpre:ENCdg?
    - WFMInpre:NR_Pt <NR1>
    - WFMInpre:NR_Pt?
    - WFMInpre:PT_Fmt {ENV|Y}
    - WFMInpre:PT_Fmt?
    - WFMInpre:PT_Off <NR1>
    - WFMInpre:PT_Off?
    - WFMInpre:REFLevel <NR3>
    - WFMInpre:REFLevel?
    - WFMInpre:SPAN <NR3>
    - WFMInpre:SPAN?
    - WFMInpre:WFMTYPe {ANALOG|RF_TD|RF_FD}
    - WFMInpre:WFMTYPe?
    - WFMInpre:XINcr <NR3>
    - WFMInpre:XINcr?
    - WFMInpre:XUNit <QString>
    - WFMInpre:XUNit?
    - WFMInpre:XZEro <NR3>
    - WFMInpre:XZEro?
    - WFMInpre:YMUlt <NR3>
    - WFMInpre:YMUlt?
    - WFMInpre:YOFf <NR3>
    - WFMInpre:YOFf?
    - WFMInpre:YUNit <QString>
    - WFMInpre:YUNit?
    - WFMInpre:YZEro <NR3>
    - WFMInpre:YZEro?
    - WFMInpre?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class WfminpreYzero(SCPICmdWrite, SCPICmdRead):
    r"""The ``WFMInpre:YZEro`` command.

    Description:
        - This command specifies the vertical offset of the destination reference waveform in units
          specified by the ``WFMInpre:YUNit`` command. Variations in this number are analogous to
          changing the vertical offset of the waveform. The ``WFMInpre:YMUlt``, ``WFMInpre:YOFf``,
          and ``WFMInpre:YZEro`` commands are used to convert waveform record values to units
          specified using the ``WFMINPRE:YUNIT`` command (YUNit units). It uses the following
          formula (where dl means digitizing levels; ``curve_in_dl`` is a data point value in the
          CURVe data): Value in YUNit units = ((``curve_in_dl`` - YOFf) \* YMUlt) + YZEro

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:YZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:YZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:YZEro value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:YZEro <NR3>
        - WFMInpre:YZEro?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the offset in in units specified by
          the ``WFMInpre:YUNit`` command (YUNits).
    """


class WfminpreYunit(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:YUNit`` command.

    Description:
        - This command specifies the vertical units of data points in the incoming waveform record
          sent to the oscilloscope using the CURVE command. This can be any of several string
          values, depending upon the vertical units of the waveform being sent. Supported units are:
          %, /Hz, A, A/A, A/V, A/W, A/dB, A/s, AA, AW, AdB, As, B, Hz, IRE, S/s, V, V/A, V/V, V/W,
          V/dB, V/s, VV, VW, VdB, volts, Vs, W, W/A, W/V, W/W, W/dB, W/s, WA, WV, WW, WdB, Ws, dB,
          dB/A, dB/V, dB/W, dB/dB, dBA, dBV, dBW, dBdB, day, degrees, div, hr, min, ohms, percent, s

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:YUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:YUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:YUNit value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:YUNit <QString>
        - WFMInpre:YUNit?
        ```

    Info:
        - ``<QString>`` contains the characters that represent the vertical units for the incoming
          waveform.
    """

    _WRAP_ARG_WITH_QUOTES = True


class WfminpreYoff(SCPICmdWrite, SCPICmdRead):
    r"""The ``WFMInpre:YOFf`` command.

    Description:
        - This command specifies the vertical position of the destination reference waveform in
          digitizing levels. There are 25 digitizing levels per vertical division for 1-byte data,
          and 6400 digitizing levels per vertical division for 2-byte data. Variations in this
          number are analogous to changing the vertical position of the waveform. The
          ``WFMInpre:YMUlt``, ``WFMInpre:YOFf``, and ``WFMInpre:YZEro`` commands are used to convert
          waveform record values to units specified using the ``WFMINPRE:YUNIT`` command (YUNit
          units). It uses the following formula (where dl means digitizing levels, and
          ``curve_in_dl`` is a data point value in the CURVe data): Value in YUNit units =
          ((``curve_in_dl`` - YOFf) \* YMUlt) + YZEro

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:YOFf?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:YOFf?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:YOFf value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:YOFf <NR3>
        - WFMInpre:YOFf?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the vertical offset in digitizing
          levels.
    """


class WfminpreYmult(SCPICmdWrite, SCPICmdRead):
    r"""The ``WFMInpre:YMUlt`` command.

    Description:
        - This command specifies the vertical scale multiplying factor to be used to convert the
          incoming data point values being sent to the oscilloscope, from digitizing levels into the
          units specified by the ``WFMInpre:YUNit`` command. For one byte waveform data, there are
          256 digitizing levels. For two byte waveform data there are 65,536 digitizing levels. The
          ``WFMInpre:YMUlt``, ``WFMInpre:YOFf``, and ``WFMInpre:YZEro`` values are used to convert
          waveform record values to units specified using the ``WFMINPRE:YUNIT`` command (YUNit
          units). It uses the following formula (where dl means digitizing level; ``curve_in_dl`` is
          a data point value in the CURVe data): Value in YUNit units = ((``curve_in_dl`` - YOFf) \*
          YMUlt) + YZEro

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:YMUlt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:YMUlt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:YMUlt value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:YMUlt <NR3>
        - WFMInpre:YMUlt?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the vertical scale factor per
          digitizing level of the incoming waveform points.
    """


class WfminpreXzero(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:XZEro`` command.

    Description:
        - This command specifies the position value of the first data point in the incoming waveform
          record being sent to the oscilloscope using the CURVE command. The units are determined or
          queried using the ``WFMInpre:XUNit`` command and are typically time, in seconds, or
          frequency, in hertz. This time or frequency is relative to the time or frequency of the
          trigger, which is always 0. Thus, the XZEro value can be negative.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:XZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:XZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:XZEro value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:XZEro <NR3>
        - WFMInpre:XZEro?
        ```

    Info:
        - ``<NR3>`` is the floating point value of the position, in XUNits, of the first sample in
          the incoming waveform. If XUNits is 's', <NR3> is the time of the first sample in the
          incoming waveform.
    """


class WfminpreXunit(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:XUNit`` command.

    Description:
        - This command specifies the horizontal units of the x-axis of the data points being sent to
          the oscilloscope using the CURVE command. This value can be in 's' for seconds, or 'Hz'.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:XUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:XUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:XUNit value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:XUNit <QString>
        - WFMInpre:XUNit?
        ```

    Info:
        - ``<QString>`` contains the characters that represent the horizontal units for the incoming
          waveform.
    """

    _WRAP_ARG_WITH_QUOTES = True


class WfminpreXincr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:XINcr`` command.

    Description:
        - This command specifies the horizontal interval between incoming waveform points sent to
          the oscilloscope using the CURVE command. The units are time, in seconds, or frequency, in
          hertz, and can be specified or queried using the ``WFMInpre:XUNit`` command.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:XINcr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:XINcr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:XINcr value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:XINcr <NR3>
        - WFMInpre:XINcr?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the horizontal interval
          representation.
    """


class WfminpreWfmtype(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:WFMTYPe`` command.

    Description:
        - This command specifies the type of waveform that is being transferred to the oscilloscope
          for storage in one of the REF1 - REF4 memory locations. The waveform type possibilities
          are the ANALOG, the RF time domain waveforms (``RF_TD``), or the RF frequency domain
          waveforms (``RF_FD``). The default is ANALOG. This parameter should be set before using
          the CURVe command to transfer a waveform from a PC to an internal reference location. The
          type of waveform that is being transferred in turn determines which window will display it
          on the instrument screen: the time domain window or frequency domain window. Both the
          analog and RF-TD arguments specify the time domain window; the ``RF_FD`` argument
          specifies the frequency domain window.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:WFMTYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:WFMTYPe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:WFMTYPe value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:WFMTYPe {ANALOG|RF_TD|RF_FD}
        - WFMInpre:WFMTYPe?
        ```

    Info:
        - ``ANALOG`` - this type includes Channels 1-4 and the Math waveform, and specifies to
          display it in the time domain window.
        - ``RF_TD`` - this type includes the three RF time domain traces (RF Amplitude vs. Time, RF
          Frequency vs. Time, and RF Phase vs. Time) and specifies to display it in the time domain
          window.
        - ``RF_FD`` - this type includes the four RF frequency domain traces (RF Normal, RF Average,
          RF Max Hold, and RF Min Hold) and specifies to display it in the frequency domain window.
    """


class WfminpreSpan(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:SPAN`` command.

    Description:
        - This command specifies the frequency span of the incoming RF trace. The span is the range
          of frequencies that can be observed around the center frequency.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:SPAN?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:SPAN?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:SPAN value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:SPAN <NR3>
        - WFMInpre:SPAN?
        ```

    Info:
        - ``<NR3>`` is a floating point number that can range from 1 kHz to 3 GHz. The maximum span
          is 3 GHz if option 3-SA3 is installed. Without that option, the maximum is identical to
          the analog bandwidth.
    """


class WfminpreReflevel(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:REFLevel`` command.

    Description:
        - This command specifies the Reference Level of the incoming waveform. This command applies
          only to frequency domain waveforms. The Reference Level is adjustable from 10 pico Watts
          (-140dBm) to 1 Watt (+30dBm).

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:REFLevel?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:REFLevel?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:REFLevel value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:REFLevel <NR3>
        - WFMInpre:REFLevel?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the Reference Level of the incoming
          waveform, in watts.
    """


class WfminprePtOff(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:PT_Off`` command.

    Description:
        - The set form of this command is ignored. The query form always returns a 0. (This command
          is listed for compatibility with other Tektronix oscilloscopes.)

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:PT_Off?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:PT_Off?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:PT_Off value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:PT_Off <NR1>
        - WFMInpre:PT_Off?
        ```
    """


class WfminprePtFmt(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:PT_Fmt`` command.

    Description:
        - This command specifies the acquisition format of the data points to be sent to the
          oscilloscope using the CURVE command. This can be Y for YT format, or ENV for envelope
          mode (min/max pairs). For YT format, each data value represents a single waveform data
          point. For envelope format, each data point represents a min/max pair, where the minimum
          value precedes the maximum value.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:PT_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:PT_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:PT_Fmt value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:PT_Fmt {ENV|Y}
        - WFMInpre:PT_Fmt?
        ```

    Info:
        - ``ENV`` specifies that the waveform is to be transmitted in envelope mode as minimum and
          maximum point pairs. Only Y values are explicitly transmitted. Absolute coordinates are
          given by.
        - ``Xn = XZEro + XINcr (n - PT_Off)``
        - ``Ynmax = YZEro + YMUlt (ynmax - YOFf)``
        - ``Ynmin = YZEro + YMUlt (ynmin - YOFf)``
        - ``Y`` specifies a normal waveform where one ASCII or binary data point is transmitted for
          each point in the waveform record. Only Y values are explicitly transmitted. Absolute
          coordinates are given by.
        - ``Xn = XZEro + XINcr (n - PT_Off)``
        - ``Yn = YZEro + YMUlt (yn - YOFf)``
    """


class WfminpreNrPt(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:NR_Pt`` command.

    Description:
        - This command specifies the number of data points that are in the incoming waveform record
          to be sent to the oscilloscope using the CURVe command.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:NR_Pt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:NR_Pt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:NR_Pt value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:NR_Pt <NR1>
        - WFMInpre:NR_Pt?
        ```

    Info:
        - ``<NR1>`` is the number of data points if ``WFMInpre:PT_Fmt`` is set to Y. It is the
          number of min-max pairs if ``WFMInpre:PT_Fmt`` is set to ENV.
    """


class WfminpreEncdg(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:ENCdg`` command.

    Description:
        - This command specifies the type of encoding of the incoming waveform data to be sent to
          the oscilloscope using the CURVe command. Supported types are BINary and ASCii.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:ENCdg?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:ENCdg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:ENCdg value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:ENCdg {ASCii|BINary}
        - WFMInpre:ENCdg?
        ```

    Info:
        - ``ASCii`` specifies that the incoming data is in ASCII format.
        - ``BINary`` specifies that the incoming data is in a binary format whose further
          interpretation requires knowledge of ``BYT_NR``, ``BIT_NR``, ``BN_FMT``, and ``BYT_OR``.
    """


class WfminpreDomain(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:DOMain`` command.

    Description:
        - This command specifies whether the information being sent to a reference location should
          be treated as integer (time domain) information, or floating point (frequency domain)
          information, for the purposes of storing the data internally. The default is TIMe. This
          parameter should be set before using the CURVe command to transfer a waveform from a PC to
          an internal reference location.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:DOMain?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:DOMain?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:DOMain value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:DOMain {TIMe|FREQuency}
        - WFMInpre:DOMain?
        ```

    Info:
        - ``TIMe`` specifies to treat the information to be transferred as integer values.
        - ``FREQuency`` specifies to treat the information to be transferred as floating point
          values.
    """


class WfminpreCenterfrequency(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:CENTERFREQuency`` command.

    Description:
        - This command specifies the center frequency of the incoming RF trace (waveform), in hertz.
          The center frequency is a precise frequency at the center of the frequency domain display.
          In many applications, it is a carrier frequency.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:CENTERFREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:CENTERFREQuency?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:CENTERFREQuency value``
          command.

    SCPI Syntax:
        ```
        - WFMInpre:CENTERFREQuency <NR3>
        - WFMInpre:CENTERFREQuency?
        ```

    Info:
        - ``<NR3>`` is a floating point number.
    """


class WfminpreBytOr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:BYT_Or`` command.

    Description:
        - This command specifies which byte of incoming binary waveform data is transmitted first
          (the byte order). The byte order can either be MSB (most significant byte first) or LSB
          (least significant byte first, also known as IBM format). This specification only has
          meaning when ``WFMINPRE:ENCDG`` is set to BINary and ``WFMINPRE:BYT_NR`` is 2.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:BYT_Or?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:BYT_Or?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:BYT_Or value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:BYT_Or {LSB|MSB}
        - WFMInpre:BYT_Or?
        ```

    Info:
        - ``LSB`` specifies that the least significant byte will be expected first.
        - ``MSB`` specifies that the most significant byte will be expected first.
    """


class WfminpreBytNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:BYT_Nr`` command.

    Description:
        - This command specifies the number of bytes per data point in the waveform data to be sent
          to the oscilloscope using the CURVe command. Changing this value also changes the value of
          ``WFMINPRE:BIT_NR``.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:BYT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:BYT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:BYT_Nr value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:BYT_Nr <NR1>
        - WFMInpre:BYT_Nr?
        ```

    Info:
        - ``<NR1>`` is the number of bytes per data point. The number of bytes can be 1 or 2 for
          Analog, Math or the digital channels D0 - D15. It can be 4 or 8 for DIGital collection
          data. It must be 4 for frequency domain trace data.
    """


class WfminpreBnFmt(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:BN_Fmt`` command.

    Description:
        - This command specifies the format of the data for outgoing waveforms when
          ``WFMInpre:ENCdg`` is set to BINary. The format can either be RI (signed integer), RP
          (positive integer) or FP (floating point).

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:BN_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:BN_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:BN_Fmt value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:BN_Fmt {RI|RP|FP}
        - WFMInpre:BN_Fmt?
        ```

    Info:
        - ``RI`` specifies signed integer data point representation.
        - ``RP`` specifies positive integer data point representation.
        - ``FP`` specifies floating point data representation.
    """


class WfminpreBitNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMInpre:BIT_Nr`` command.

    Description:
        - This command specifies the number of bits per data point in the waveform data to be sent
          to the oscilloscope using the CURVe command. Changing this value also changes the value of
          ``WFMINPRE:BYT_NR``.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre:BIT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre:BIT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMInpre:BIT_Nr value`` command.

    SCPI Syntax:
        ```
        - WFMInpre:BIT_Nr <NR1>
        - WFMInpre:BIT_Nr?
        ```

    Info:
        - ``<NR1>`` number of bits per data point can be 8 , 16, 32 or 64.
    """


#  pylint: disable=too-many-instance-attributes
class Wfminpre(SCPICmdRead):
    """The ``WFMInpre`` command.

    Description:
        - Returns the waveform formatting and scaling specifications to be applied to the next
          incoming CURVe command data.

    Usage:
        - Using the ``.query()`` method will send the ``WFMInpre?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMInpre?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMInpre?
        ```

    Properties:
        - ``.bit_nr``: The ``WFMInpre:BIT_Nr`` command.
        - ``.bn_fmt``: The ``WFMInpre:BN_Fmt`` command.
        - ``.byt_nr``: The ``WFMInpre:BYT_Nr`` command.
        - ``.byt_or``: The ``WFMInpre:BYT_Or`` command.
        - ``.centerfrequency``: The ``WFMInpre:CENTERFREQuency`` command.
        - ``.domain``: The ``WFMInpre:DOMain`` command.
        - ``.encdg``: The ``WFMInpre:ENCdg`` command.
        - ``.nr_pt``: The ``WFMInpre:NR_Pt`` command.
        - ``.pt_fmt``: The ``WFMInpre:PT_Fmt`` command.
        - ``.pt_off``: The ``WFMInpre:PT_Off`` command.
        - ``.reflevel``: The ``WFMInpre:REFLevel`` command.
        - ``.span``: The ``WFMInpre:SPAN`` command.
        - ``.wfmtype``: The ``WFMInpre:WFMTYPe`` command.
        - ``.xincr``: The ``WFMInpre:XINcr`` command.
        - ``.xunit``: The ``WFMInpre:XUNit`` command.
        - ``.xzero``: The ``WFMInpre:XZEro`` command.
        - ``.ymult``: The ``WFMInpre:YMUlt`` command.
        - ``.yoff``: The ``WFMInpre:YOFf`` command.
        - ``.yunit``: The ``WFMInpre:YUNit`` command.
        - ``.yzero``: The ``WFMInpre:YZEro`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "WFMInpre") -> None:
        super().__init__(device, cmd_syntax)
        self._bit_nr = WfminpreBitNr(device, f"{self._cmd_syntax}:BIT_Nr")
        self._bn_fmt = WfminpreBnFmt(device, f"{self._cmd_syntax}:BN_Fmt")
        self._byt_nr = WfminpreBytNr(device, f"{self._cmd_syntax}:BYT_Nr")
        self._byt_or = WfminpreBytOr(device, f"{self._cmd_syntax}:BYT_Or")
        self._centerfrequency = WfminpreCenterfrequency(
            device, f"{self._cmd_syntax}:CENTERFREQuency"
        )
        self._domain = WfminpreDomain(device, f"{self._cmd_syntax}:DOMain")
        self._encdg = WfminpreEncdg(device, f"{self._cmd_syntax}:ENCdg")
        self._nr_pt = WfminpreNrPt(device, f"{self._cmd_syntax}:NR_Pt")
        self._pt_fmt = WfminprePtFmt(device, f"{self._cmd_syntax}:PT_Fmt")
        self._pt_off = WfminprePtOff(device, f"{self._cmd_syntax}:PT_Off")
        self._reflevel = WfminpreReflevel(device, f"{self._cmd_syntax}:REFLevel")
        self._span = WfminpreSpan(device, f"{self._cmd_syntax}:SPAN")
        self._wfmtype = WfminpreWfmtype(device, f"{self._cmd_syntax}:WFMTYPe")
        self._xincr = WfminpreXincr(device, f"{self._cmd_syntax}:XINcr")
        self._xunit = WfminpreXunit(device, f"{self._cmd_syntax}:XUNit")
        self._xzero = WfminpreXzero(device, f"{self._cmd_syntax}:XZEro")
        self._ymult = WfminpreYmult(device, f"{self._cmd_syntax}:YMUlt")
        self._yoff = WfminpreYoff(device, f"{self._cmd_syntax}:YOFf")
        self._yunit = WfminpreYunit(device, f"{self._cmd_syntax}:YUNit")
        self._yzero = WfminpreYzero(device, f"{self._cmd_syntax}:YZEro")

    @property
    def bit_nr(self) -> WfminpreBitNr:
        """Return the ``WFMInpre:BIT_Nr`` command.

        Description:
            - This command specifies the number of bits per data point in the waveform data to be
              sent to the oscilloscope using the CURVe command. Changing this value also changes the
              value of ``WFMINPRE:BYT_NR``.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:BIT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:BIT_Nr?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:BIT_Nr value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:BIT_Nr <NR1>
            - WFMInpre:BIT_Nr?
            ```

        Info:
            - ``<NR1>`` number of bits per data point can be 8 , 16, 32 or 64.
        """
        return self._bit_nr

    @property
    def bn_fmt(self) -> WfminpreBnFmt:
        """Return the ``WFMInpre:BN_Fmt`` command.

        Description:
            - This command specifies the format of the data for outgoing waveforms when
              ``WFMInpre:ENCdg`` is set to BINary. The format can either be RI (signed integer), RP
              (positive integer) or FP (floating point).

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:BN_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:BN_Fmt?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:BN_Fmt value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:BN_Fmt {RI|RP|FP}
            - WFMInpre:BN_Fmt?
            ```

        Info:
            - ``RI`` specifies signed integer data point representation.
            - ``RP`` specifies positive integer data point representation.
            - ``FP`` specifies floating point data representation.
        """
        return self._bn_fmt

    @property
    def byt_nr(self) -> WfminpreBytNr:
        """Return the ``WFMInpre:BYT_Nr`` command.

        Description:
            - This command specifies the number of bytes per data point in the waveform data to be
              sent to the oscilloscope using the CURVe command. Changing this value also changes the
              value of ``WFMINPRE:BIT_NR``.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:BYT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:BYT_Nr?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:BYT_Nr value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:BYT_Nr <NR1>
            - WFMInpre:BYT_Nr?
            ```

        Info:
            - ``<NR1>`` is the number of bytes per data point. The number of bytes can be 1 or 2 for
              Analog, Math or the digital channels D0 - D15. It can be 4 or 8 for DIGital collection
              data. It must be 4 for frequency domain trace data.
        """
        return self._byt_nr

    @property
    def byt_or(self) -> WfminpreBytOr:
        """Return the ``WFMInpre:BYT_Or`` command.

        Description:
            - This command specifies which byte of incoming binary waveform data is transmitted
              first (the byte order). The byte order can either be MSB (most significant byte first)
              or LSB (least significant byte first, also known as IBM format). This specification
              only has meaning when ``WFMINPRE:ENCDG`` is set to BINary and ``WFMINPRE:BYT_NR`` is
              2.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:BYT_Or?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:BYT_Or?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:BYT_Or value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:BYT_Or {LSB|MSB}
            - WFMInpre:BYT_Or?
            ```

        Info:
            - ``LSB`` specifies that the least significant byte will be expected first.
            - ``MSB`` specifies that the most significant byte will be expected first.
        """
        return self._byt_or

    @property
    def centerfrequency(self) -> WfminpreCenterfrequency:
        """Return the ``WFMInpre:CENTERFREQuency`` command.

        Description:
            - This command specifies the center frequency of the incoming RF trace (waveform), in
              hertz. The center frequency is a precise frequency at the center of the frequency
              domain display. In many applications, it is a carrier frequency.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:CENTERFREQuency?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:CENTERFREQuency?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:CENTERFREQuency value``
              command.

        SCPI Syntax:
            ```
            - WFMInpre:CENTERFREQuency <NR3>
            - WFMInpre:CENTERFREQuency?
            ```

        Info:
            - ``<NR3>`` is a floating point number.
        """
        return self._centerfrequency

    @property
    def domain(self) -> WfminpreDomain:
        """Return the ``WFMInpre:DOMain`` command.

        Description:
            - This command specifies whether the information being sent to a reference location
              should be treated as integer (time domain) information, or floating point (frequency
              domain) information, for the purposes of storing the data internally. The default is
              TIMe. This parameter should be set before using the CURVe command to transfer a
              waveform from a PC to an internal reference location.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:DOMain?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:DOMain?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:DOMain value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:DOMain {TIMe|FREQuency}
            - WFMInpre:DOMain?
            ```

        Info:
            - ``TIMe`` specifies to treat the information to be transferred as integer values.
            - ``FREQuency`` specifies to treat the information to be transferred as floating point
              values.
        """
        return self._domain

    @property
    def encdg(self) -> WfminpreEncdg:
        """Return the ``WFMInpre:ENCdg`` command.

        Description:
            - This command specifies the type of encoding of the incoming waveform data to be sent
              to the oscilloscope using the CURVe command. Supported types are BINary and ASCii.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:ENCdg?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:ENCdg?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:ENCdg value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:ENCdg {ASCii|BINary}
            - WFMInpre:ENCdg?
            ```

        Info:
            - ``ASCii`` specifies that the incoming data is in ASCII format.
            - ``BINary`` specifies that the incoming data is in a binary format whose further
              interpretation requires knowledge of ``BYT_NR``, ``BIT_NR``, ``BN_FMT``, and
              ``BYT_OR``.
        """
        return self._encdg

    @property
    def nr_pt(self) -> WfminpreNrPt:
        """Return the ``WFMInpre:NR_Pt`` command.

        Description:
            - This command specifies the number of data points that are in the incoming waveform
              record to be sent to the oscilloscope using the CURVe command.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:NR_Pt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:NR_Pt?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:NR_Pt value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:NR_Pt <NR1>
            - WFMInpre:NR_Pt?
            ```

        Info:
            - ``<NR1>`` is the number of data points if ``WFMInpre:PT_Fmt`` is set to Y. It is the
              number of min-max pairs if ``WFMInpre:PT_Fmt`` is set to ENV.
        """
        return self._nr_pt

    @property
    def pt_fmt(self) -> WfminprePtFmt:
        """Return the ``WFMInpre:PT_Fmt`` command.

        Description:
            - This command specifies the acquisition format of the data points to be sent to the
              oscilloscope using the CURVE command. This can be Y for YT format, or ENV for envelope
              mode (min/max pairs). For YT format, each data value represents a single waveform data
              point. For envelope format, each data point represents a min/max pair, where the
              minimum value precedes the maximum value.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:PT_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:PT_Fmt?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:PT_Fmt value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:PT_Fmt {ENV|Y}
            - WFMInpre:PT_Fmt?
            ```

        Info:
            - ``ENV`` specifies that the waveform is to be transmitted in envelope mode as minimum
              and maximum point pairs. Only Y values are explicitly transmitted. Absolute
              coordinates are given by.
            - ``Xn = XZEro + XINcr (n - PT_Off)``
            - ``Ynmax = YZEro + YMUlt (ynmax - YOFf)``
            - ``Ynmin = YZEro + YMUlt (ynmin - YOFf)``
            - ``Y`` specifies a normal waveform where one ASCII or binary data point is transmitted
              for each point in the waveform record. Only Y values are explicitly transmitted.
              Absolute coordinates are given by.
            - ``Xn = XZEro + XINcr (n - PT_Off)``
            - ``Yn = YZEro + YMUlt (yn - YOFf)``
        """
        return self._pt_fmt

    @property
    def pt_off(self) -> WfminprePtOff:
        """Return the ``WFMInpre:PT_Off`` command.

        Description:
            - The set form of this command is ignored. The query form always returns a 0. (This
              command is listed for compatibility with other Tektronix oscilloscopes.)

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:PT_Off?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:PT_Off?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:PT_Off value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:PT_Off <NR1>
            - WFMInpre:PT_Off?
            ```
        """
        return self._pt_off

    @property
    def reflevel(self) -> WfminpreReflevel:
        """Return the ``WFMInpre:REFLevel`` command.

        Description:
            - This command specifies the Reference Level of the incoming waveform. This command
              applies only to frequency domain waveforms. The Reference Level is adjustable from 10
              pico Watts (-140dBm) to 1 Watt (+30dBm).

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:REFLevel?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:REFLevel?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:REFLevel value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:REFLevel <NR3>
            - WFMInpre:REFLevel?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the Reference Level of the
              incoming waveform, in watts.
        """
        return self._reflevel

    @property
    def span(self) -> WfminpreSpan:
        """Return the ``WFMInpre:SPAN`` command.

        Description:
            - This command specifies the frequency span of the incoming RF trace. The span is the
              range of frequencies that can be observed around the center frequency.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:SPAN?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:SPAN?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:SPAN value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:SPAN <NR3>
            - WFMInpre:SPAN?
            ```

        Info:
            - ``<NR3>`` is a floating point number that can range from 1 kHz to 3 GHz. The maximum
              span is 3 GHz if option 3-SA3 is installed. Without that option, the maximum is
              identical to the analog bandwidth.
        """
        return self._span

    @property
    def wfmtype(self) -> WfminpreWfmtype:
        """Return the ``WFMInpre:WFMTYPe`` command.

        Description:
            - This command specifies the type of waveform that is being transferred to the
              oscilloscope for storage in one of the REF1 - REF4 memory locations. The waveform type
              possibilities are the ANALOG, the RF time domain waveforms (``RF_TD``), or the RF
              frequency domain waveforms (``RF_FD``). The default is ANALOG. This parameter should
              be set before using the CURVe command to transfer a waveform from a PC to an internal
              reference location. The type of waveform that is being transferred in turn determines
              which window will display it on the instrument screen: the time domain window or
              frequency domain window. Both the analog and RF-TD arguments specify the time domain
              window; the ``RF_FD`` argument specifies the frequency domain window.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:WFMTYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:WFMTYPe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:WFMTYPe value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:WFMTYPe {ANALOG|RF_TD|RF_FD}
            - WFMInpre:WFMTYPe?
            ```

        Info:
            - ``ANALOG`` - this type includes Channels 1-4 and the Math waveform, and specifies to
              display it in the time domain window.
            - ``RF_TD`` - this type includes the three RF time domain traces (RF Amplitude vs. Time,
              RF Frequency vs. Time, and RF Phase vs. Time) and specifies to display it in the time
              domain window.
            - ``RF_FD`` - this type includes the four RF frequency domain traces (RF Normal, RF
              Average, RF Max Hold, and RF Min Hold) and specifies to display it in the frequency
              domain window.
        """
        return self._wfmtype

    @property
    def xincr(self) -> WfminpreXincr:
        """Return the ``WFMInpre:XINcr`` command.

        Description:
            - This command specifies the horizontal interval between incoming waveform points sent
              to the oscilloscope using the CURVE command. The units are time, in seconds, or
              frequency, in hertz, and can be specified or queried using the ``WFMInpre:XUNit``
              command.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:XINcr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:XINcr?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:XINcr value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:XINcr <NR3>
            - WFMInpre:XINcr?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the horizontal interval
              representation.
        """
        return self._xincr

    @property
    def xunit(self) -> WfminpreXunit:
        """Return the ``WFMInpre:XUNit`` command.

        Description:
            - This command specifies the horizontal units of the x-axis of the data points being
              sent to the oscilloscope using the CURVE command. This value can be in 's' for
              seconds, or 'Hz'.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:XUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:XUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:XUNit value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:XUNit <QString>
            - WFMInpre:XUNit?
            ```

        Info:
            - ``<QString>`` contains the characters that represent the horizontal units for the
              incoming waveform.
        """
        return self._xunit

    @property
    def xzero(self) -> WfminpreXzero:
        """Return the ``WFMInpre:XZEro`` command.

        Description:
            - This command specifies the position value of the first data point in the incoming
              waveform record being sent to the oscilloscope using the CURVE command. The units are
              determined or queried using the ``WFMInpre:XUNit`` command and are typically time, in
              seconds, or frequency, in hertz. This time or frequency is relative to the time or
              frequency of the trigger, which is always 0. Thus, the XZEro value can be negative.

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:XZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:XZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:XZEro value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:XZEro <NR3>
            - WFMInpre:XZEro?
            ```

        Info:
            - ``<NR3>`` is the floating point value of the position, in XUNits, of the first sample
              in the incoming waveform. If XUNits is 's', <NR3> is the time of the first sample in
              the incoming waveform.
        """
        return self._xzero

    @property
    def ymult(self) -> WfminpreYmult:
        r"""Return the ``WFMInpre:YMUlt`` command.

        Description:
            - This command specifies the vertical scale multiplying factor to be used to convert the
              incoming data point values being sent to the oscilloscope, from digitizing levels into
              the units specified by the ``WFMInpre:YUNit`` command. For one byte waveform data,
              there are 256 digitizing levels. For two byte waveform data there are 65,536
              digitizing levels. The ``WFMInpre:YMUlt``, ``WFMInpre:YOFf``, and ``WFMInpre:YZEro``
              values are used to convert waveform record values to units specified using the
              ``WFMINPRE:YUNIT`` command (YUNit units). It uses the following formula (where dl
              means digitizing level; ``curve_in_dl`` is a data point value in the CURVe data):
              Value in YUNit units = ((``curve_in_dl`` - YOFf) \* YMUlt) + YZEro

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:YMUlt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:YMUlt?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:YMUlt value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:YMUlt <NR3>
            - WFMInpre:YMUlt?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the vertical scale factor per
              digitizing level of the incoming waveform points.
        """
        return self._ymult

    @property
    def yoff(self) -> WfminpreYoff:
        r"""Return the ``WFMInpre:YOFf`` command.

        Description:
            - This command specifies the vertical position of the destination reference waveform in
              digitizing levels. There are 25 digitizing levels per vertical division for 1-byte
              data, and 6400 digitizing levels per vertical division for 2-byte data. Variations in
              this number are analogous to changing the vertical position of the waveform. The
              ``WFMInpre:YMUlt``, ``WFMInpre:YOFf``, and ``WFMInpre:YZEro`` commands are used to
              convert waveform record values to units specified using the ``WFMINPRE:YUNIT`` command
              (YUNit units). It uses the following formula (where dl means digitizing levels, and
              ``curve_in_dl`` is a data point value in the CURVe data): Value in YUNit units =
              ((``curve_in_dl`` - YOFf) \* YMUlt) + YZEro

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:YOFf?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:YOFf?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:YOFf value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:YOFf <NR3>
            - WFMInpre:YOFf?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the vertical offset in digitizing
              levels.
        """
        return self._yoff

    @property
    def yunit(self) -> WfminpreYunit:
        """Return the ``WFMInpre:YUNit`` command.

        Description:
            - This command specifies the vertical units of data points in the incoming waveform
              record sent to the oscilloscope using the CURVE command. This can be any of several
              string values, depending upon the vertical units of the waveform being sent. Supported
              units are: %, /Hz, A, A/A, A/V, A/W, A/dB, A/s, AA, AW, AdB, As, B, Hz, IRE, S/s, V,
              V/A, V/V, V/W, V/dB, V/s, VV, VW, VdB, volts, Vs, W, W/A, W/V, W/W, W/dB, W/s, WA, WV,
              WW, WdB, Ws, dB, dB/A, dB/V, dB/W, dB/dB, dBA, dBV, dBW, dBdB, day, degrees, div, hr,
              min, ohms, percent, s

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:YUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:YUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:YUNit value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:YUNit <QString>
            - WFMInpre:YUNit?
            ```

        Info:
            - ``<QString>`` contains the characters that represent the vertical units for the
              incoming waveform.
        """
        return self._yunit

    @property
    def yzero(self) -> WfminpreYzero:
        r"""Return the ``WFMInpre:YZEro`` command.

        Description:
            - This command specifies the vertical offset of the destination reference waveform in
              units specified by the ``WFMInpre:YUNit`` command. Variations in this number are
              analogous to changing the vertical offset of the waveform. The ``WFMInpre:YMUlt``,
              ``WFMInpre:YOFf``, and ``WFMInpre:YZEro`` commands are used to convert waveform record
              values to units specified using the ``WFMINPRE:YUNIT`` command (YUNit units). It uses
              the following formula (where dl means digitizing levels; ``curve_in_dl`` is a data
              point value in the CURVe data): Value in YUNit units = ((``curve_in_dl`` - YOFf) \*
              YMUlt) + YZEro

        Usage:
            - Using the ``.query()`` method will send the ``WFMInpre:YZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMInpre:YZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMInpre:YZEro value`` command.

        SCPI Syntax:
            ```
            - WFMInpre:YZEro <NR3>
            - WFMInpre:YZEro?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the offset in in units specified
              by the ``WFMInpre:YUNit`` command (YUNits).
        """
        return self._yzero



# pyright: reportConstantRedefinition=none
"""The digio commands module.

These commands are used in the following models:
MP5103

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - digio.readbit()
    - digio.readport()
    - digio.trigger[N].assert()
    - digio.trigger[N].edge
    - digio.trigger[N].logic
    - digio.trigger[N].mode
    - digio.trigger[N].overrun
    - digio.trigger[N].pulsewidth
    - digio.trigger[N].release()
    - digio.trigger[N].reset()
    - digio.trigger[N].stimulus
    - digio.trigger[N].wait()
    - digio.writebit()
    - digio.writeport()
    - digio.writeprotect
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING, Union

from ..helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class DigioTriggerItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``digio.trigger[N]`` command tree.

    Info:
        - ``N``, the digital I/O trigger line: 1 to 18.

    Constants:
        - ``.EVENT_ID``: The trigger event generated by the digital I/O line N.

    Properties and methods:
        - ``.assert()``: The ``digio.trigger[N].assert()`` function.
        - ``.edge``: The ``digio.trigger[N].edge`` attribute.
        - ``.logic``: The ``digio.trigger[N].logic`` attribute.
        - ``.mode``: The ``digio.trigger[N].mode`` attribute.
        - ``.overrun``: The ``digio.trigger[N].overrun`` attribute.
        - ``.pulsewidth``: The ``digio.trigger[N].pulsewidth`` attribute.
        - ``.release()``: The ``digio.trigger[N].release()`` function.
        - ``.reset()``: The ``digio.trigger[N].reset()`` function.
        - ``.stimulus``: The ``digio.trigger[N].stimulus`` attribute.
        - ``.wait()``: The ``digio.trigger[N].wait()`` function.
    """

    EVENT_ID = "digio.trigger[N].EVENT_ID"
    """str: The trigger event generated by the digital I/O line N."""

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )

    @property
    def edge(self) -> str:
        """Access the ``digio.trigger[N].edge`` attribute.

        Description:
            - This command configures the edge detection mode for a digital I/O line.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].edge)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].edge = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].edge = value
            - print(digio.trigger[N].edge)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".edge"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.edge)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.edge`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @edge.setter
    def edge(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].edge`` attribute.

        Description:
            - This command configures the edge detection mode for a digital I/O line.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].edge)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].edge = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].edge = value
            - print(digio.trigger[N].edge)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".edge", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.edge = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.edge`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def logic(self) -> str:
        """Access the ``digio.trigger[N].logic`` attribute.

        Description:
            - This attribute sets the output logic of the trigger event generator to positive or
              negative for the specified line.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].logic)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].logic = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].logic = value
            - print(digio.trigger[N].logic)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".logic"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.logic)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.logic`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @logic.setter
    def logic(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].logic`` attribute.

        Description:
            - This attribute sets the output logic of the trigger event generator to positive or
              negative for the specified line.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].logic)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].logic = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].logic = value
            - print(digio.trigger[N].logic)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".logic", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.logic = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.logic`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def mode(self) -> str:
        """Access the ``digio.trigger[N].mode`` attribute.

        Description:
            - This attribute sets the mode of the digital I/O line to be a digital line, trigger
              line, or synchronous line and sets the line to be input, output, or open-drain.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].mode = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].mode = value
            - print(digio.trigger[N].mode)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.mode)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].mode`` attribute.

        Description:
            - This attribute sets the mode of the digital I/O line to be a digital line, trigger
              line, or synchronous line and sets the line to be input, output, or open-drain.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].mode = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].mode = value
            - print(digio.trigger[N].mode)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.mode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def overrun(self) -> str:
        """Access the ``digio.trigger[N].overrun`` attribute.

        Description:
            - This function returns the event detector overrun status.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].overrun)`` query.

        TSP Syntax:
            ```
            - print(digio.trigger[N].overrun)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".overrun"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.overrun)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.overrun`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def pulsewidth(self) -> str:
        """Access the ``digio.trigger[N].pulsewidth`` attribute.

        Description:
            - This attribute describes the length of time that the trigger line is asserted for
              output triggers.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].pulsewidth)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].pulsewidth = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].pulsewidth = value
            - print(digio.trigger[N].pulsewidth)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".pulsewidth"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.pulsewidth)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.pulsewidth`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @pulsewidth.setter
    def pulsewidth(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].pulsewidth`` attribute.

        Description:
            - This attribute describes the length of time that the trigger line is asserted for
              output triggers.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].pulsewidth)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].pulsewidth = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].pulsewidth = value
            - print(digio.trigger[N].pulsewidth)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".pulsewidth", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.pulsewidth = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.pulsewidth`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def stimulus(self) -> str:
        """Access the ``digio.trigger[N].stimulus`` attribute.

        Description:
            - This attribute selects the event that causes a trigger to be asserted on the digital
              output line.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].stimulus = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].stimulus = value
            - print(digio.trigger[N].stimulus)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``digio.trigger[N].stimulus`` attribute.

        Description:
            - This attribute selects the event that causes a trigger to be asserted on the digital
              output line.

        Usage:
            - Accessing this property will send the ``print(digio.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``digio.trigger[N].stimulus = value``
              command.

        TSP Syntax:
            ```
            - digio.trigger[N].stimulus = value
            - print(digio.trigger[N].stimulus)
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def assert_(self) -> None:
        """Run the ``digio.trigger[N].assert()`` function.

        Description:
            - This function asserts a trigger pulse on one of the digital I/O lines

        TSP Syntax:
            ```
            - digio.trigger[N].assert()
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.assert()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.assert()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def release(self) -> None:
        """Run the ``digio.trigger[N].release()`` function.

        Description:
            - This function releases an indefinite length or latched trigger.

        TSP Syntax:
            ```
            - digio.trigger[N].release()
            ```

        Info:
            - ``N``, the digital I/O trigger line (1 to 18).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.release()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.release()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self) -> None:
        """Run the ``digio.trigger[N].reset()`` function.

        Description:
            - This function resets digital I/O line values to their factory defaults.

        TSP Syntax:
            ```
            - digio.trigger[N].reset()
            ```

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.reset()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def wait(self, timeout: float) -> str:
        """Run the ``digio.trigger[N].wait()`` function.

        Description:
            - This function waits for an input trigger.

        TSP Syntax:
            ```
            - digio.trigger[N].wait()
            ```

        Args:
            timeout: Timeout in seconds.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.wait({timeout}))"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.wait()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Digio(BaseTSPCmd):
    """The ``digio`` command tree.

    Properties and methods:
        - ``.readbit()``: The ``digio.readbit()`` function.
        - ``.readport()``: The ``digio.readport()`` function.
        - ``.trigger``: The ``digio.trigger[N]`` command tree.
        - ``.writebit()``: The ``digio.writebit()`` function.
        - ``.writeport()``: The ``digio.writeport()`` function.
        - ``.writeprotect``: The ``digio.writeprotect`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "digio") -> None:
        super().__init__(device, cmd_syntax)
        self._trigger: Dict[int, DigioTriggerItem] = DefaultDictPassKeyToFactory(
            lambda x: DigioTriggerItem(device, f"{self._cmd_syntax}.trigger[{x}]")
        )

    @property
    def trigger(self) -> Dict[int, DigioTriggerItem]:
        """Return the ``digio.trigger[N]`` command tree.

        Info:
            - ``N``, the digital I/O trigger line: 1 to 18.

        Constants:
            - ``.EVENT_ID``: The trigger event generated by the digital I/O line N.

        Sub-properties and sub-methods:
            - ``.assert()``: The ``digio.trigger[N].assert()`` function.
            - ``.edge``: The ``digio.trigger[N].edge`` attribute.
            - ``.logic``: The ``digio.trigger[N].logic`` attribute.
            - ``.mode``: The ``digio.trigger[N].mode`` attribute.
            - ``.overrun``: The ``digio.trigger[N].overrun`` attribute.
            - ``.pulsewidth``: The ``digio.trigger[N].pulsewidth`` attribute.
            - ``.release()``: The ``digio.trigger[N].release()`` function.
            - ``.reset()``: The ``digio.trigger[N].reset()`` function.
            - ``.stimulus``: The ``digio.trigger[N].stimulus`` attribute.
            - ``.wait()``: The ``digio.trigger[N].wait()`` function.
        """
        return self._trigger

    @property
    def writeprotect(self) -> str:
        """Access the ``digio.writeprotect`` attribute.

        Description:
            - This attribute contains the write-protect mask that protects bits from changes from
              the digio.writebit() and digio.writeport() functions.

        Usage:
            - Accessing this property will send the ``print(digio.writeprotect)`` query.
            - Setting this property to a value will send the ``digio.writeprotect = value`` command.

        TSP Syntax:
            ```
            - digio.writeprotect = value
            - print(digio.writeprotect)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".writeprotect"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.writeprotect)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.writeprotect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @writeprotect.setter
    def writeprotect(self, value: Union[str, float]) -> None:
        """Access the ``digio.writeprotect`` attribute.

        Description:
            - This attribute contains the write-protect mask that protects bits from changes from
              the digio.writebit() and digio.writeport() functions.

        Usage:
            - Accessing this property will send the ``print(digio.writeprotect)`` query.
            - Setting this property to a value will send the ``digio.writeprotect = value`` command.

        TSP Syntax:
            ```
            - digio.writeprotect = value
            - print(digio.writeprotect)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".writeprotect", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.writeprotect = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.writeprotect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readbit(self, n: str) -> str:
        """Run the ``digio.readbit()`` function.

        Description:
            - This function reads one digital I/O line.

        TSP Syntax:
            ```
            - digio.readbit()
            ```

        Args:
            n: Digital I/O line number to be read.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readbit({n}))"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.readbit()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readport(self) -> str:
        """Run the ``digio.readport()`` function.

        Description:
            - This function reads the digital I/O port.

        TSP Syntax:
            ```
            - digio.readport()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readport())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.readport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writebit(self, n: str, value: int) -> None:
        """Run the ``digio.writebit()`` function.

        Description:
            - This function writes a value to a specific digital I/O line.

        TSP Syntax:
            ```
            - digio.writebit()
            ```

        Args:
            n: Digital I/O trigger line.
            value: The value to write to the bit.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.writebit({n}, {value})"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.writebit()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writeport(self, data: int) -> None:
        """Run the ``digio.writeport()`` function.

        Description:
            - This function writes to all digital I/O lines.

        TSP Syntax:
            ```
            - digio.writeport()
            ```

        Args:
            data: Value to write to the port.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.writeport({data})"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.writeport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



# pyright: reportConstantRedefinition=none
"""The userstring commands module.

These commands are used in the following models:
DAQ6510, DMM6500, DMM7510, MP5103, SMU2450, SMU2460, SMU2461, SMU2470, SMU2601B, SMU2602B, SMU2604B,
SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B, SMU2651A, SMU2657A, SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - userstring.add()
    - userstring.delete()
    - userstring.get()
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Userstring(BaseTSPCmd):
    """The ``userstring`` command tree.

    Properties and methods:
        - ``.add()``: The ``userstring.add()`` function.
        - ``.delete()``: The ``userstring.delete()`` function.
        - ``.get()``: The ``userstring.get()`` function.
    """

    def __init__(
        self, device: Optional["TSPControl"] = None, cmd_syntax: str = "userstring"
    ) -> None:
        super().__init__(device, cmd_syntax)

    def add(self, name: str, value: str) -> None:
        """Run the ``userstring.add()`` function.

        Description:
            - This function adds a user-defined string to nonvolatile memory.

        TSP Syntax:
            ```
            - userstring.add()
            ```

        Args:
            name: The name of the string; the key of the key-value pair.
            value: The string to associate with name; the value of the key-value pair.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.add("{name}", "{value}")'
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.add()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def delete(self, name: str) -> None:
        """Run the ``userstring.delete()`` function.

        Description:
            - This function deletes a user-defined string from nonvolatile memory.

        TSP Syntax:
            ```
            - userstring.delete()
            ```

        Args:
            name: The name (key) of the key-value pair of the user-defined string to delete.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.delete("{name}")'
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.delete()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def get(self, name: str) -> str:
        """Run the ``userstring.get()`` function.

        Description:
            - This function retrieves a user-defined string from nonvolatile memory.

        TSP Syntax:
            ```
            - userstring.get()
            ```

        Args:
            name: The name (key) of the user-defined string.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.get("{name}"))'
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.get()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The time commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - TIMe <QString>
    - TIMe:ZONe <QString>
    - TIMe:ZONe:UTCDELTa <NR3>
    - TIMe:ZONe:UTCDELTa?
    - TIMe:ZONe?
    - TIMe?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class TimeZoneUtcdelta(SCPICmdWrite, SCPICmdRead):
    """The ``TIMe:ZONe:UTCDELTa`` command.

    Description:
        - This command sets or queries the time zone using the difference between the desired time
          zone and UTC.

    Usage:
        - Using the ``.query()`` method will send the ``TIMe:ZONe:UTCDELTa?`` query.
        - Using the ``.verify(value)`` method will send the ``TIMe:ZONe:UTCDELTa?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TIMe:ZONe:UTCDELTa value`` command.

    SCPI Syntax:
        ```
        - TIMe:ZONe:UTCDELTa <NR3>
        - TIMe:ZONe:UTCDELTa?
        ```

    Info:
        - ``<NR3>`` is the specified number of hours difference between the desired time zoneand UTC
          which is equivalent to GMT. The deltas supported are: -12.00, -11.00,-10.00, -9.30, -9.00,
          -8.30, -8.00, -7.00, -6.00, -5.00, -4.00, -3.30, -3.00, -2.00,-1.00, 0.0, 1.00, 2.00,
          3.00, 3.30, 4.00, 4.30, 5.0.
    """


class TimeZone(SCPICmdWrite, SCPICmdRead):
    """The ``TIMe:ZONe`` command.

    Description:
        - This command sets the time zone to the one specified.

    Usage:
        - Using the ``.query()`` method will send the ``TIMe:ZONe?`` query.
        - Using the ``.verify(value)`` method will send the ``TIMe:ZONe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TIMe:ZONe value`` command.

    SCPI Syntax:
        ```
        - TIMe:ZONe <QString>
        - TIMe:ZONe?
        ```

    Info:
        - ``<QString>`` is a quoted string representing the desired time.

    Properties:
        - ``.utcdelta``: The ``TIMe:ZONe:UTCDELTa`` command.
    """

    _WRAP_ARG_WITH_QUOTES = True

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._utcdelta = TimeZoneUtcdelta(device, f"{self._cmd_syntax}:UTCDELTa")

    @property
    def utcdelta(self) -> TimeZoneUtcdelta:
        """Return the ``TIMe:ZONe:UTCDELTa`` command.

        Description:
            - This command sets or queries the time zone using the difference between the desired
              time zone and UTC.

        Usage:
            - Using the ``.query()`` method will send the ``TIMe:ZONe:UTCDELTa?`` query.
            - Using the ``.verify(value)`` method will send the ``TIMe:ZONe:UTCDELTa?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TIMe:ZONe:UTCDELTa value`` command.

        SCPI Syntax:
            ```
            - TIMe:ZONe:UTCDELTa <NR3>
            - TIMe:ZONe:UTCDELTa?
            ```

        Info:
            - ``<NR3>`` is the specified number of hours difference between the desired time zoneand
              UTC which is equivalent to GMT. The deltas supported are: -12.00, -11.00,-10.00,
              -9.30, -9.00, -8.30, -8.00, -7.00, -6.00, -5.00, -4.00, -3.30, -3.00, -2.00,-1.00,
              0.0, 1.00, 2.00, 3.00, 3.30, 4.00, 4.30, 5.0.
        """
        return self._utcdelta


class Time(SCPICmdWrite, SCPICmdRead):
    """The ``TIMe`` command.

    Description:
        - This command sets the time in the form ``hh:mm:ss`` where hh refers to a two-digit hour
          number, mm refers to a two-digit minute number from 01 to 60, and ss refers to a two-digit
          second number from 01 to 60.

    Usage:
        - Using the ``.query()`` method will send the ``TIMe?`` query.
        - Using the ``.verify(value)`` method will send the ``TIMe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TIMe value`` command.

    SCPI Syntax:
        ```
        - TIMe <QString>
        - TIMe?
        ```

    Info:
        - ``<QString>`` is a quoted string representing the desired time.

    Properties:
        - ``.zone``: The ``TIMe:ZONe`` command.
    """

    _WRAP_ARG_WITH_QUOTES = True

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "TIMe") -> None:
        super().__init__(device, cmd_syntax)
        self._zone = TimeZone(device, f"{self._cmd_syntax}:ZONe")

    @property
    def zone(self) -> TimeZone:
        """Return the ``TIMe:ZONe`` command.

        Description:
            - This command sets the time zone to the one specified.

        Usage:
            - Using the ``.query()`` method will send the ``TIMe:ZONe?`` query.
            - Using the ``.verify(value)`` method will send the ``TIMe:ZONe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TIMe:ZONe value`` command.

        SCPI Syntax:
            ```
            - TIMe:ZONe <QString>
            - TIMe:ZONe?
            ```

        Info:
            - ``<QString>`` is a quoted string representing the desired time.

        Sub-properties:
            - ``.utcdelta``: The ``TIMe:ZONe:UTCDELTa`` command.
        """
        return self._zone



"""The evmsg commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC,
DSA70KC, DSA70KD, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4K, MSO4KB, MSO5K,
MSO5KB, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - EVMsg?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Evmsg(SCPICmdRead):
    """The ``EVMsg`` command.

    Description:
        - This query-only command removes a single event code from the Event Queue that is
          associated with the results of the last ESR read and returns the event code with an
          explanatory message. For more information, see Event Handling.

    Usage:
        - Using the ``.query()`` method will send the ``EVMsg?`` query.
        - Using the ``.verify(value)`` method will send the ``EVMsg?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EVMsg?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "EVMsg") -> None:
        super().__init__(device, cmd_syntax)



"""The meastable commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO2, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MEASTABle:ADDNew <QString>
    - MEASTABle:DELETE <QString>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MeastableDelete(SCPICmdWrite):
    """The ``MEASTABle:DELETE`` command.

    Description:
        - This command removes the requested measurement results view table from the scope
          application.

    Usage:
        - Using the ``.write(value)`` method will send the ``MEASTABle:DELETE value`` command.

    SCPI Syntax:
        ```
        - MEASTABle:DELETE <QString>
        ```

    Info:
        - ``<QString>`` specifies a measurement results view table to remove.
    """

    _WRAP_ARG_WITH_QUOTES = True


class MeastableAddnew(SCPICmdWrite):
    """The ``MEASTABle:ADDNew`` command.

    Description:
        - This command adds a new measurement results view table to the scope application. The
          results view table can be named through the argument sent to the command.

    Usage:
        - Using the ``.write(value)`` method will send the ``MEASTABle:ADDNew value`` command.

    SCPI Syntax:
        ```
        - MEASTABle:ADDNew <QString>
        ```

    Info:
        - ``<QString>`` specifies a new measurement results view table to the scope application.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Meastable(SCPICmdRead):
    """The ``MEASTABle`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MEASTABle?`` query.
        - Using the ``.verify(value)`` method will send the ``MEASTABle?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.addnew``: The ``MEASTABle:ADDNew`` command.
        - ``.delete``: The ``MEASTABle:DELETE`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "MEASTABle") -> None:
        super().__init__(device, cmd_syntax)
        self._addnew = MeastableAddnew(device, f"{self._cmd_syntax}:ADDNew")
        self._delete = MeastableDelete(device, f"{self._cmd_syntax}:DELETE")

    @property
    def addnew(self) -> MeastableAddnew:
        """Return the ``MEASTABle:ADDNew`` command.

        Description:
            - This command adds a new measurement results view table to the scope application. The
              results view table can be named through the argument sent to the command.

        Usage:
            - Using the ``.write(value)`` method will send the ``MEASTABle:ADDNew value`` command.

        SCPI Syntax:
            ```
            - MEASTABle:ADDNew <QString>
            ```

        Info:
            - ``<QString>`` specifies a new measurement results view table to the scope application.
        """
        return self._addnew

    @property
    def delete(self) -> MeastableDelete:
        """Return the ``MEASTABle:DELETE`` command.

        Description:
            - This command removes the requested measurement results view table from the scope
              application.

        Usage:
            - Using the ``.write(value)`` method will send the ``MEASTABle:DELETE value`` command.

        SCPI Syntax:
            ```
            - MEASTABle:DELETE <QString>
            ```

        Info:
            - ``<QString>`` specifies a measurement results view table to remove.
        """
        return self._delete



"""The customtable commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CUSTOMTABle:ADDNew <QString>
    - CUSTOMTABle:DELete <QString>
    - CUSTOMTABle:LIST? <QString>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdReadWithArguments, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class CustomtableList(SCPICmdReadWithArguments):
    """The ``CUSTOMTABle:LIST`` command.

    Description:
        - This command queries the list of custom result tables has been added.

    Usage:
        - Using the ``.query(argument)`` method will send the ``CUSTOMTABle:LIST? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``CUSTOMTABle:LIST? argument``
          query and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CUSTOMTABle:LIST? <QString>
        ```

    Info:
        - ``<QString>`` specifies the custom results table name.
    """


class CustomtableDelete(SCPICmdWrite):
    """The ``CUSTOMTABle:DELete`` command.

    Description:
        - This command deletes the custom result(s) table that was added.

    Usage:
        - Using the ``.write(value)`` method will send the ``CUSTOMTABle:DELete value`` command.

    SCPI Syntax:
        ```
        - CUSTOMTABle:DELete <QString>
        ```

    Info:
        - ``<QString>`` specifies the custom results table name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class CustomtableAddnew(SCPICmdWrite):
    """The ``CUSTOMTABle:ADDNew`` command.

    Description:
        - This command adds new custom results table.

    Usage:
        - Using the ``.write(value)`` method will send the ``CUSTOMTABle:ADDNew value`` command.

    SCPI Syntax:
        ```
        - CUSTOMTABle:ADDNew <QString>
        ```

    Info:
        - ``<QString>`` specifies the custom results table name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Customtable(SCPICmdRead):
    """The ``CUSTOMTABle`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CUSTOMTABle?`` query.
        - Using the ``.verify(value)`` method will send the ``CUSTOMTABle?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.addnew``: The ``CUSTOMTABle:ADDNew`` command.
        - ``.delete``: The ``CUSTOMTABle:DELete`` command.
        - ``.list``: The ``CUSTOMTABle:LIST`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "CUSTOMTABle"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._addnew = CustomtableAddnew(device, f"{self._cmd_syntax}:ADDNew")
        self._delete = CustomtableDelete(device, f"{self._cmd_syntax}:DELete")
        self._list = CustomtableList(device, f"{self._cmd_syntax}:LIST")

    @property
    def addnew(self) -> CustomtableAddnew:
        """Return the ``CUSTOMTABle:ADDNew`` command.

        Description:
            - This command adds new custom results table.

        Usage:
            - Using the ``.write(value)`` method will send the ``CUSTOMTABle:ADDNew value`` command.

        SCPI Syntax:
            ```
            - CUSTOMTABle:ADDNew <QString>
            ```

        Info:
            - ``<QString>`` specifies the custom results table name.
        """
        return self._addnew

    @property
    def delete(self) -> CustomtableDelete:
        """Return the ``CUSTOMTABle:DELete`` command.

        Description:
            - This command deletes the custom result(s) table that was added.

        Usage:
            - Using the ``.write(value)`` method will send the ``CUSTOMTABle:DELete value`` command.

        SCPI Syntax:
            ```
            - CUSTOMTABle:DELete <QString>
            ```

        Info:
            - ``<QString>`` specifies the custom results table name.
        """
        return self._delete

    @property
    def list(self) -> CustomtableList:
        """Return the ``CUSTOMTABle:LIST`` command.

        Description:
            - This command queries the list of custom result tables has been added.

        Usage:
            - Using the ``.query(argument)`` method will send the ``CUSTOMTABle:LIST? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``CUSTOMTABle:LIST? argument`` query and raise an AssertionError if the returned value
              does not match ``value``.

        SCPI Syntax:
            ```
            - CUSTOMTABle:LIST? <QString>
            ```

        Info:
            - ``<QString>`` specifies the custom results table name.
        """
        return self._list



"""A collection of classes and other helpful objects to use when generating TSP drivers.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""

from typing import Any, Dict, Optional, TYPE_CHECKING

from .generic_commands import BaseCmd

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


####################################################################################################
# Classes
####################################################################################################
class BaseTSPCmd(BaseCmd):  # pylint: disable=too-few-public-methods
    """A class to better type hint a member of a TSP command tree."""

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._device: Optional["TSPControl"] = device

    def _get_tsp_table_dict(self, table: str) -> Dict[Any, Any]:
        """Get the full content of a TSP table variable.

        Args:
            table: The name of the TSP table variable

        Returns:
            A dict with all the table content.
        """
        keys = self._device.query(  # type: ignore[union-attr]
            f's="" for k,v in pairs({table}) do s = s..k.."; " end print(s)'
        ).split("; ")
        table_dict: Dict[Any, Any] = {}
        for key in filter(None, keys):
            if (
                self._device.query(f"print(type({table}.{key}))")  # type: ignore[union-attr]
                == "table"
            ):
                table_dict[key] = self._get_tsp_table_dict(f"{table}.{key}")
            else:
                table_dict[key] = self._device.query(  # type: ignore[union-attr]
                    f"print({table}.{key})"
                )
        return table_dict



"""The multiscope commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MULTiscope:CONFig {ATI|TEKCONnect|TIMESYnc}
    - MULTiscope:EXIT
    - MULTiscope:RESTART
    - MULTiscope:STATUS?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MultiscopeStatus(SCPICmdRead):
    """The ``MULTiscope:STATUS`` command.

    Description:
        - This query returns the current state of oscilloscopes in the MultiScope system.

    Usage:
        - Using the ``.query()`` method will send the ``MULTiscope:STATUS?`` query.
        - Using the ``.verify(value)`` method will send the ``MULTiscope:STATUS?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MULTiscope:STATUS?
        ```
    """


class MultiscopeRestart(SCPICmdWriteNoArguments):
    """The ``MULTiscope:RESTART`` command.

    Description:
        - This command instructs the Configuration Managers (and active scope applications) across
          the entire MultiScope configuration to exit and restart themselves.

    Usage:
        - Using the ``.write()`` method will send the ``MULTiscope:RESTART`` command.

    SCPI Syntax:
        ```
        - MULTiscope:RESTART
        ```
    """


class MultiscopeExit(SCPICmdWriteNoArguments):
    """The ``MULTiscope:EXIT`` command.

    Description:
        - This command instructs the Configuration Managers (and active scope applications) across
          the entire MultiScope configuration to shutdown.

    Usage:
        - Using the ``.write()`` method will send the ``MULTiscope:EXIT`` command.

    SCPI Syntax:
        ```
        - MULTiscope:EXIT
        ```
    """


class MultiscopeConfig(SCPICmdWrite):
    """The ``MULTiscope:CONFig`` command.

    Description:
        - This command sets or queries the current MultiScope operational configuration. This
          command is for the Master instrument in the multi-instrument configuration.

    Usage:
        - Using the ``.write(value)`` method will send the ``MULTiscope:CONFig value`` command.

    SCPI Syntax:
        ```
        - MULTiscope:CONFig {ATI|TEKCONnect|TIMESYnc}
        ```

    Info:
        - ``ATI`` sets the MultiScope system to use the ATI channels.
        - ``TEKCONNECT`` sets the MultiScope system to use the TekConnect channels.
        - ``TIMESYNC`` sets the MultiScope system to use the TekConnect channels in TimeSync mode.
          TimeSync mode is only available on some instruments. The Master instrument controls the
          instrument hardware such as the horizontal and trigger settings. Desired results are only
          achieved in Single Sequence acquisition mode. Data from all channels can be acquired using
          programmable interface commands.
    """


class Multiscope(SCPICmdRead):
    """The ``MULTiscope`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MULTiscope?`` query.
        - Using the ``.verify(value)`` method will send the ``MULTiscope?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.config``: The ``MULTiscope:CONFig`` command.
        - ``.exit``: The ``MULTiscope:EXIT`` command.
        - ``.restart``: The ``MULTiscope:RESTART`` command.
        - ``.status``: The ``MULTiscope:STATUS`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "MULTiscope"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._config = MultiscopeConfig(device, f"{self._cmd_syntax}:CONFig")
        self._exit = MultiscopeExit(device, f"{self._cmd_syntax}:EXIT")
        self._restart = MultiscopeRestart(device, f"{self._cmd_syntax}:RESTART")
        self._status = MultiscopeStatus(device, f"{self._cmd_syntax}:STATUS")

    @property
    def config(self) -> MultiscopeConfig:
        """Return the ``MULTiscope:CONFig`` command.

        Description:
            - This command sets or queries the current MultiScope operational configuration. This
              command is for the Master instrument in the multi-instrument configuration.

        Usage:
            - Using the ``.write(value)`` method will send the ``MULTiscope:CONFig value`` command.

        SCPI Syntax:
            ```
            - MULTiscope:CONFig {ATI|TEKCONnect|TIMESYnc}
            ```

        Info:
            - ``ATI`` sets the MultiScope system to use the ATI channels.
            - ``TEKCONNECT`` sets the MultiScope system to use the TekConnect channels.
            - ``TIMESYNC`` sets the MultiScope system to use the TekConnect channels in TimeSync
              mode. TimeSync mode is only available on some instruments. The Master instrument
              controls the instrument hardware such as the horizontal and trigger settings. Desired
              results are only achieved in Single Sequence acquisition mode. Data from all channels
              can be acquired using programmable interface commands.
        """
        return self._config

    @property
    def exit(self) -> MultiscopeExit:
        """Return the ``MULTiscope:EXIT`` command.

        Description:
            - This command instructs the Configuration Managers (and active scope applications)
              across the entire MultiScope configuration to shutdown.

        Usage:
            - Using the ``.write()`` method will send the ``MULTiscope:EXIT`` command.

        SCPI Syntax:
            ```
            - MULTiscope:EXIT
            ```
        """
        return self._exit

    @property
    def restart(self) -> MultiscopeRestart:
        """Return the ``MULTiscope:RESTART`` command.

        Description:
            - This command instructs the Configuration Managers (and active scope applications)
              across the entire MultiScope configuration to exit and restart themselves.

        Usage:
            - Using the ``.write()`` method will send the ``MULTiscope:RESTART`` command.

        SCPI Syntax:
            ```
            - MULTiscope:RESTART
            ```
        """
        return self._restart

    @property
    def status(self) -> MultiscopeStatus:
        """Return the ``MULTiscope:STATUS`` command.

        Description:
            - This query returns the current state of oscilloscopes in the MultiScope system.

        Usage:
            - Using the ``.query()`` method will send the ``MULTiscope:STATUS?`` query.
            - Using the ``.verify(value)`` method will send the ``MULTiscope:STATUS?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MULTiscope:STATUS?
            ```
        """
        return self._status
